/*
 *
 *  	gtkseti (C) 1999 Fabio Cavaliere (fabio@urania.fisica.unige.it) 
 *  
 *      GUI made in glade.   
 *
 * 	This program is free software; you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation; either version 2, or (at your option)
 * 	any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program (see the file COPYING); if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 *      Boston, MA  02111-1307, USA
 *
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h> 
#include <stdlib.h>
#include <string.h>

#include "interface.h"
#include "support.h"
#include "callbacks.h"

#define RUNNING 0
#define STOPPED 1
#define KILLED 2
#define SLEEPING 3

GtkWidget *label4;
GtkWidget *label5;
GtkWidget *label6;
GtkWidget *label10;
GtkWidget *label11;
GtkWidget *label12;
GtkWidget *progressbar1;
GtkWidget *progressbar2;
GtkWidget *principale;
GtkWidget *clist1;
GtkWidget *hiscore;

char path[512],setidir[512],Path[512],Pref[512],Uinfo[512],Hiscore[512],Rcfile[512];
int PID,status,NL,TIMER,FILEISOPEN,AUTORUN,AUTOKILL,SHOWN;
long int t2,t1;
float dt;
double overall_bs,overall_bg,overall_time;
FILE *f1,*history_in,*history_out;

void getpath(void);
int scanstate(gpointer data);

int
main (int argc, char *argv[])
{
  GtkWidget *principale;
  char dummy[512],dummy1[512],dummy2[512],*entry,*value;
  int i,n;
  FILE *fin,*fout;

  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  sprintf(setidir,"setiathome");
  dt = 5;	
  NL = 19;
  
  AUTORUN = 0;
  AUTOKILL = 0;
  SHOWN = 0;

  getpath();
  
  printf("\n");
  printf("---------------------------------------------------------------\n");
  printf(" Welcome to GtkSeti 0.1 - A gtk front end for Seti@home client\n");
  printf(" By Fabio Cavaliere - This software is free under GPL license.\n");
  printf(" \tInfo : fabio@urania.fisica.unige.it\n");
  printf("---------------------------------------------------------------\n\n");
  
  /*
   * Parse .gtksetirc.
   *
   * This code fragment is the same as in scansate() . It's not very sure, for 
   * the syntax has to be "VARIABLE=VALUE", once per line, otherwise everything
   * could happen. Anyway, it should simply skip wrong entries !   
   *
   * Currently supported options :
   *
   * 	nice={-19...19} : sets nicelevel on autorun (only works with autorun).
   *	delay={2.0,+oo}	: delay between subsequent updates (in seconds).
   * 	autorun={0,1}	: 1 means gtkseti will try to run client at startup.
   *    autokill={0,1}  : 1 means gtkseti will kill client on exit.
   *    setidir={DIR}   : gtkseti will use DIR as working directory, relative 
   *			  to your home directory.
   */

  if ((f1=fopen(Rcfile,"r")) != NULL) {
  	do {
  		fscanf(f1,"%s\n",(char *)&dummy);
  		entry=strtok(dummy,"=");
  		value=strtok(NULL,"=");
  		if (entry != NULL && value != NULL) {
  			if (!strcmp(dummy,"nice")) NL=(int)strtod(value,0);
  			if (!strcmp(dummy,"autorun")) AUTORUN=(int)strtod(value,0);
  			if (!strcmp(dummy,"autokill")) AUTOKILL=(int)strtod(value,0);
  			if (!strcmp(dummy,"delay")) dt=(float)strtod(value,0);
  			if (!strcmp(dummy,"setidir")) strcpy(setidir,value);
  			}
  		} while(!feof(f1));
  	fclose(f1);
 	}	

  getpath();
  
  /*
   * Check for user_defined parameter bonduaries.
   */

  if (NL < -19 || NL > 19) NL = 19;
  if (dt < 2.0) dt = 5.0; /* It could hang on slower machines... */
  if (AUTORUN < 0 || AUTORUN > 1) AUTORUN = 0;
  if (AUTOKILL < 0 || AUTOKILL > 1) AUTORUN = 0;
  if (AUTORUN != 1) NL = 19;
  
  add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
  add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");

  /*
   * Check for work directory.
   */

  if ((f1=fopen(Uinfo,"r")) == NULL) {
  	gtk_widget_show(create_Splash1());
  	gtk_main();
  	system("rm --force /tmp/.gtksetilock");
	return -1;
  	}
   fclose(f1);
  
  /*
   * Prevent running more than one instance of GtkSeti.
   */
  
  if ((f1=fopen("/tmp/.gtksetilock","r")) != NULL) {
	gtk_widget_show(create_Splash2());
	gtk_main();
	fclose(f1);
	return -1;
	}  
  		
  /*
   * To do this, we use a simple lock file...
   */  
  
  system("touch /tmp/.gtksetilock");
  
  principale = create_principale ();
  gtk_widget_show (principale);

  /*
   * Let's do a backup copy of history and "highscore" files. In case of crash
   * they could be lost, so...
   */
 
  n=0;
  sprintf(dummy1,"%s/.gtkseti",getenv("HOME"));
  sprintf(dummy2,"%s/.gtkseti_bak",getenv("HOME"));
  if ((fin=fopen(dummy1,"r")) != NULL) {
  	do {
  		fgets(dummy,512,fin);
  		n++;
 		} while(!feof(fin));
 	fin=fopen(dummy1,"r");	 		
  	fout=fopen(dummy2,"w");
  	for (i=1;i <= n-1;i++) {
  		fgets(dummy,512,fin);
  		fprintf(fout,"%s",dummy);
  		} 
	fclose(fin);
	fclose(fout);
  	}
  
  n=0;
  sprintf(dummy1,"%s/seti_highscore",getenv("HOME"));
  sprintf(dummy2,"%s/seti_highscore.bak",getenv("HOME"));
  if ((fin=fopen(dummy1,"r")) != NULL) {
  	do {
  		fgets(dummy,512,fin);
		n++;
		} while(!feof(fin));
  	fin=fopen(dummy1,"r");
  	fout=fopen(dummy2,"w");
  	for (i=1;i <= n-1;i++) {
  		fgets(dummy,512,fin);
  		fprintf(fout,"%s",dummy);
  		} 
  	fclose(fin);
  	fclose(fout);
  	}

  /*
   * Read history...
   */
 
  if ((history_in=fopen(Pref,"r")) != NULL) {
  	fscanf(history_in,"%lf\n%lf\n%lf\n",&overall_bg,&overall_bs,&overall_time);
  	fclose(history_in);
  	}
  if (history_in == NULL) {
  	overall_bs=0.0;
  	overall_bg=0.0;
  	overall_time=0.0;
  	}
  
  FILEISOPEN = 0;
  
  if (AUTORUN == 1) {
  	sprintf(dummy,"cd %s ; setiathome -email -nice %d > /dev/null &",path,NL);
  	system(dummy);
  	}
    
  gtk_timeout_add(dt*1000.0,scanstate,NULL);
  TIMER = 0;
  scanstate(NULL);
  TIMER = 1;

  gtk_main ();
 
  /*
   * Shouldn't happen, anyway...
   */
  
  if (FILEISOPEN == 1) fclose(history_out);
  
  /*
   * Remove lock file.
   */
  
  system("rm --force /tmp/.gtksetilock");
  
  if (AUTOKILL == 1 && status != KILLED) {
  	sprintf(dummy,"kill -9 %d",PID);
  	system(dummy);
  	}
  	
  return 0;
}
